/* Copyright (c) 2004 Marijn Haverbeke
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any
 * damages arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any
 * purpose, including commercial applications, and to alter it and
 * redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software. If you use this
 *    software in a product, an acknowledgment in the product
 *    documentation would be appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must
 *    not be misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source
 *    distribution.
 *
 * Marijn Haverbeke
 * marijn(at)haverbeke.nl
 */

// System to generate the default background. Tries to do some kind of
// fractal brownian motion thing. Not very well done but it works.

#include <cstring>
#include <cstdlib>
#include <wx/image.h>
#include "background.hpp"

namespace{
  const size_t random_size = 8;

  // Array (8x8) to get repeating random numbers. I doctored these a
  // little to remove outliers. This whole file is one big hack,
  // nothing elegant or reusable here.
  const unsigned char random_nums[] = {
    167, 141, 217, 42, 130, 80, 116, 174, 67, 77, 152, 85, 140, 226, 179, 71, 223, 47, 152, 84, 47,
    117, 45, 75, 88, 245, 107, 214, 136, 87, 153, 146, 72, 51, 98, 65, 143, 193, 35, 169, 95, 48, 109, 200,
    237, 97, 212, 75, 92, 53, 57, 129, 132, 184, 20, 162, 156, 180, 90, 103, 42, 102, 149, 72
  };

  // Extract a number from the array.
  int Random_Array(int x, int y)
  {
    return random_nums[(x % random_size) + (y % random_size) * random_size];
  }

  // Interpolate between two numbers.
  inline float Interpolate(float a, float b, float part)
  {
    return a * part + b * (1.f - part);
  }
  inline float Interpolate(int a, int b, float part)
  {
    return Interpolate(static_cast<float>(a), static_cast<float>(b), part);
  }

  // Interpolates a value in 2d for point x,y if the current octave
  // has a wavelength of scale
  float Get_Value(int x, int y, int scale)
  {
    // The x and y at the start of this wave
    int x2 = x / scale;
    int y2 = y / scale;

    // Portion of this wave that x,y lies on
    float x_part = static_cast<float>(x % scale) / scale;
    float y_part = static_cast<float>(y % scale) / scale;
    // This is for smooth interpolation, makes the shapes look less
    // diamond-like, which is what you get with 2d linear
    // interpolation
    x_part = x_part * x_part * (3 - 2 * x_part);
    y_part = y_part * y_part * (3 - 2 * y_part);

    // Compute the values above and below x,y and interpolate them
    float top = Interpolate(Random_Array(x2 + 1, y2), Random_Array(x2, y2), x_part);
    float bottom = Interpolate(Random_Array(x2 + 1, y2 + 1), Random_Array(x2, y2 + 1), x_part);
    float center = Interpolate(bottom, top, y_part);
    
    return center;
  }

  // Size of the background image
  const size_t background_size = 256;

  // Add an octave to an array (yes, I chose some really meaningful
  // names for functions here)
  void Increase_Array(unsigned char* array, int scale, float r, float g, float b)
  {
    // For every pixel
    for (size_t y = 0; y != background_size; ++y)
    {
      for (size_t x = 0; x != background_size; ++x)
      {
        // Increase the red, green and blue values of that for this
        // octave
        float value = Get_Value(x, y, scale);
        array[0] += static_cast<unsigned char>(value * r);
        array[1] += static_cast<unsigned char>(value * g);
        array[2] += static_cast<unsigned char>(value * b);
        array += 3;
      }
    }
  }
}

// Create the default background - a blueish noise
wxImage Get_Default_Background()
{
  // First make an array for the picture data and set it all to 20
  // (dark grey). The reason I'm using malloc here is that the wxImage
  // will get ownership of this data and will delete it with free.
  unsigned char* data = reinterpret_cast<unsigned char*>(std::malloc(background_size * background_size * 3));
  std::memset(reinterpret_cast<void*>(data), 20, background_size * background_size * 3);

  // Add 4 octaves
  Increase_Array(data, 32, .2, .2, .25);
  Increase_Array(data, 16, .16, .12, .12);
  Increase_Array(data, 8, .1, .07, .08);
  Increase_Array(data, 4, .04, .03, .06);

  // Create an image
  return wxImage(background_size, background_size, data);
}
