/* Copyright (c) 2004 Marijn Haverbeke
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any
 * damages arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any
 * purpose, including commercial applications, and to alter it and
 * redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software. If you use this
 *    software in a product, an acknowledgment in the product
 *    documentation would be appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must
 *    not be misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source
 *    distribution.
 *
 * Marijn Haverbeke
 * marijn(at)haverbeke.nl
 */

// Main.cpp - main application and frame.

#include <ctime>
#include <cstdlib>
#include <wx/dirdlg.h>
#define wxUSE_COLOURDLG 1
#include <wx/colordlg.h>
#include <wx/image.h>
#include "main.hpp"
#include "canvas.hpp"
#include "jigsaw.hpp"
#include "ids.hpp"
#include "sizedialog.hpp"
#include "background.hpp"

namespace{
  // Some big strings that are defined at the bottom of this file.
  extern const char* license;
  extern const char* help;

  // Ask the user for an image file.
  wxString Get_Image_File(const wxString& dialog_caption)
  {
    const wxString file_extensions = "Image files (.bmp, .gif, .jpg, .png, .pcx)|*.bmp;*.gif;*.jpg;*.png;*.pcx";
    return wxFileSelector(dialog_caption, "", "", file_extensions, file_extensions, wxOPEN);
  }
}

IMPLEMENT_APP(Jigsaw_App)

// Initialize the application
bool Jigsaw_App::OnInit()
{
  // Create and show the main frame
  Main_Frame* frame = new Main_Frame();
  frame->Show(true);
  SetTopWindow(frame);

  // Initialize image handlers (or wx won't load images)
  wxInitAllImageHandlers();
  // Seed random number generator
  std::srand(std::time(NULL));
  
  return true;
}

// Event table for the main frame, all of them menu actions
BEGIN_EVENT_TABLE(Main_Frame, wxFrame)
  EVT_MENU(id_menu_new, Main_Frame::New_Game)
  EVT_MENU(id_menu_close, Main_Frame::Close_Game)
  EVT_MENU(id_menu_quit, Main_Frame::Quit)
  EVT_MENU(id_menu_background_default, Main_Frame::Set_Default_Background)
  EVT_MENU(id_menu_background_color, Main_Frame::Set_Background_Color)
  EVT_MENU(id_menu_background_image, Main_Frame::Set_Background_Image)
  EVT_MENU(id_menu_help, Main_Frame::Help)
  EVT_MENU(id_menu_license, Main_Frame::License)
  EVT_MENU(id_menu_about, Main_Frame::About)
  END_EVENT_TABLE();

// The main frame does not have a lot of responsibilities, exept for
// housing the menus. Direct manipulation of pieces is handled by the
// canvas.
Main_Frame::Main_Frame()
    // A frame of 800 by 600 pixels, maximized if possible.
  : wxFrame(NULL, id_main, "OpenJigsaw v" VERSION, wxPoint(0, 0), wxSize(800, 600), wxMAXIMIZE | wxDEFAULT_FRAME_STYLE),
    // The canvas handles all the dragging and drawing.
    _canvas(new Canvas(this)),
    _game(NULL)
{
  // Set the window icon.
#ifdef __WINDOWS__
  wxIcon icon("MAIN_ICON", wxBITMAP_TYPE_ICO_RESOURCE);
  SetIcon(icon);
#endif

  // Create the menus
  wxMenu* file_menu = new wxMenu;
  file_menu->Append(id_menu_new, "&New puzzle");
  file_menu->Append(id_menu_close, "&Close puzzle");
  file_menu->Append(id_menu_quit, "E&xit");

  wxMenu* background_menu = new wxMenu;
  background_menu->Append(id_menu_background_default, "&Default");
  background_menu->Append(id_menu_background_color, "Solid &color");
  background_menu->Append(id_menu_background_image, "&Image");

  wxMenu* help_menu = new wxMenu;
  help_menu->Append(id_menu_help, "&Help");
  help_menu->Append(id_menu_license, "&License");
  help_menu->Append(id_menu_about, "&About");
  
  wxMenuBar* menu_bar = new wxMenuBar;
  menu_bar->Append(file_menu, "&File");
  menu_bar->Append(background_menu, "&Background");
  menu_bar->Append(help_menu, "&Help");
  SetMenuBar(menu_bar);

  // Give the canvas it's initial background
  _canvas->Set_Background(Get_Default_Background());
}

Main_Frame::~Main_Frame()
{
  delete _game;
}

void Main_Frame::Quit(wxCommandEvent& event)
{
  Close(true);
}

void Main_Frame::Help(wxCommandEvent& event)
{
  wxMessageBox(help, "OpenJigsaw Help", wxOK, this);
}

void Main_Frame::License(wxCommandEvent& event)
{
  wxMessageBox(license, "OpenJigsaw License", wxOK, this);
}

void Main_Frame::About(wxCommandEvent& event)
{
  wxMessageBox("This is version " VERSION " of OpenJigsaw\nWritten by Marijn Haverbeke\nhttp://marijn.haverbeke.nl",
               "About OpenJigsaw", wxOK | wxICON_INFORMATION, this);
}

void Main_Frame::Set_Default_Background(wxCommandEvent& event)
{
  _canvas->Set_Background(Get_Default_Background());
}

void Main_Frame::Set_Background_Color(wxCommandEvent& event)
{
  // Query the user for a color using the convenient wx color dialog,
  // and pass it to the canvas if a valid color was given.
  wxColor color = wxGetColourFromUser(this, *wxWHITE);
  if (color.Ok())
    _canvas->Set_Background(color);
}

void Main_Frame::Set_Background_Image(wxCommandEvent& event)
{
  // Ask user for a file, if one is given and it yields a proper image
  // we pass it to the canvas.
  wxString filename = Get_Image_File("Select background image");
  if (!filename.empty())
  {
    wxImage image(filename);
    if (image.Ok())
      _canvas->Set_Background(image);
  }
}

void Main_Frame::New_Game(wxCommandEvent& event)
{
  // Get a file
  wxString filename = Get_Image_File("Select image for this puzzle");
  if (filename.empty())
    return;

  // See if it is a valid image
  wxImage image(filename);
  if (!image.Ok())
    return;

  // Get a size for this image
  wxPoint size = Select_Puzzle_Size(image.GetWidth(), image.GetHeight());
  if (size.x == 0)
    return;

  // Delete the old game and make a new game
  delete _game;
  _game = new Jigsaw(image, size.x, size.y);
  _canvas->Set_Game(_game);
}

// Close the current game
void Main_Frame::Close_Game(wxCommandEvent& event)
{
  delete _game;
  _game = NULL;
  _canvas->Set_Game(NULL);
}

wxPoint Main_Frame::Select_Puzzle_Size(int image_width, int image_height)
{
  // Calculate the height/width ratio for the image
  float ratio = static_cast<float>(image_height) / image_width;

  // Query the user for a puzzle size
  Size_Dialog* dialog = new Size_Dialog(ratio);
  int width = dialog->ShowModal();
  dialog->Destroy();

  return wxPoint(width, Compute_Height(width, ratio));
}

namespace{
  // These actually display quite ugly in a dialog because the
  // newlines should be in other places in a variable-width font. I
  // was too lazy to do something about that.
  const char* license =
    "   Copyright (c) 2004 Marijn Haverbeke\n"
    "\n"
    "This software is provided 'as-is', without any express or implied\n"
    "warranty. In no event will the authors be held liable for any damages\n"
    "arising from the use of this software.\n"
    "\n"
    "Permission is granted to anyone to use this software for any purpose,\n"
    "including commercial applications, and to alter it and redistribute it\n"
    "freely, subject to the following restrictions:\n"
    "\n"
    "1. The origin of this software must not be misrepresented; you must not\n"
    "     claim that you wrote the original software. If you use this software\n"
    "     in a product, an acknowledgment in the product documentation would\n"
    "     be appreciated but is not required.\n"
    "\n"
    "2. Altered source versions must be plainly marked as such, and must not\n"
    "     be misrepresented as being the original software.\n"
    "\n"
    "3. This notice may not be removed or altered from any source\n"
    "     distribution.\n"
    "\n"
    "Source available at: http://marijn.haverbeke.nl/jigsaw\n"
    "Marijn Haverbeke\n"
    "marijn(at)haverbeke.nl\n";

  const char* help =
    "OpenJigsaw is a program that converts images into jigsaw puzzles and\n"
    "lets you solve them. Choose 'new puzzle' from the 'file' menu to start\n"
    "on a puzzle. You will be asked to pick an image and a size for the\n"
    "jigsaw and the pieces will appear.\n"
    "\n"
    "Move pieces by dragging them with the left mouse button. You can drag\n"
    "with the right mouse button to clear pieces off a portion of the\n"
    "screen. If the background color is too similar to some of the pieces\n"
    "it can be hard to see them. In that case, use the 'background' menu to\n"
    "pick another color or image to be used as a background.\n";
}
    
