/* Copyright (c) 2004 Marijn Haverbeke
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any
 * damages arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any
 * purpose, including commercial applications, and to alter it and
 * redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software. If you use this
 *    software in a product, an acknowledgment in the product
 *    documentation would be appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must
 *    not be misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source
 *    distribution.
 *
 * Marijn Haverbeke
 * marijn(at)haverbeke.nl
 */

// A size dialog is displayed when a new puzzle is being made, it
// displays a number of possible sizes and lets the user choose one.

#include <vector>
#include "sizedialog.hpp"
#include "ids.hpp"

BEGIN_EVENT_TABLE(Size_Dialog, wxWindow)
  EVT_LISTBOX_DCLICK(id_size_list, Size_Dialog::Ok)
  EVT_BUTTON(id_size_ok, Size_Dialog::Ok)
  EVT_BUTTON(id_size_cancel, Size_Dialog::Cancel)
  END_EVENT_TABLE();

// The possible widths. You can add to this to allow more different
// sizes, but a puzzle 15 pieces wide will fill most monitors. The 0
// is the terminator of the array.
const size_t widths[] = {4, 7, 10, 12, 15, 0};

// Create a dialog and add all the stuff inside it
Size_Dialog::Size_Dialog(float size_ratio)
  : wxDialog(NULL, id_size_dialog, "Puzzle size", wxDefaultPosition, wxDefaultSize),
    _list_box(NULL)
{
  // The space between the widgets.
  const int dialog_margin = 5;

  // Create the text for the choices
  std::vector<wxString> choices;
  for (const size_t* width = widths; *width != 0; ++width)
  {
    size_t height = Compute_Height(*width, size_ratio);
    choices.push_back(wxString());
    choices.back() << *width * height << " pieces (" << *width << 'x' << height << ')';
  }

  // The widgets: a caption on top, then the listbox, then an OK
  // button and a cancel button
  wxStaticText* caption = new wxStaticText(this, id_size_caption, "Choose the number of pieces for\nthis puzzle.", wxDefaultPosition, wxDefaultSize);
  _list_box = new wxListBox(this, id_size_list, wxDefaultPosition, wxDefaultSize, choices.size(), &choices[0], wxLB_SINGLE | wxLB_NEEDED_SB);
  wxButton* ok_button = new wxButton(this, id_size_ok, "&OK", wxDefaultPosition, wxDefaultSize); 
  wxButton* cancel_button = new wxButton(this, id_size_cancel, "&Cancel", wxDefaultPosition, wxDefaultSize); 

  // Put the buttons next to each other in a horizontal sizer
  wxBoxSizer* button_sizer = new wxBoxSizer(wxHORIZONTAL);
  button_sizer->Add(dialog_margin, 0);
  button_sizer->Add(ok_button, 0, wxTOP | wxBOTTOM, dialog_margin);
  button_sizer->Add(dialog_margin, 0);
  button_sizer->Add(cancel_button, 0, wxTOP | wxBOTTOM, dialog_margin);
  button_sizer->Add(dialog_margin, 0);

  // Add everything to a vertical sizer. This is not as pretty as I'd
  // have liked it to be - the buttons centered and the listbox
  // growing to fill the frame. When I do any of those things stuff
  // goes all wrong in wxGTK (works in MSW). Not sure whether this is
  // a bug or my fault.
  wxBoxSizer* main_sizer = new wxBoxSizer(wxVERTICAL);
  main_sizer->Add(0, dialog_margin);
  main_sizer->Add(caption, 0, wxLEFT | wxRIGHT, dialog_margin);
  main_sizer->Add(0, dialog_margin);
  main_sizer->Add(_list_box, 0, wxLEFT | wxRIGHT, dialog_margin);
  main_sizer->Add(button_sizer, 0);

  // Connect sizer to dialog.
  SetSizer(main_sizer);
  main_sizer->SetSizeHints(this);
  main_sizer->Layout();

  // Select first item in the list
  _list_box->SetSelection(0);
}

// OK pressed or double clicked in listbox
void Size_Dialog::Ok(wxCommandEvent& event)
{
  // Only return if something is selected
  int selected = _list_box->GetSelection();
  if (selected != -1)
    EndModal(widths[selected]);
}

// Cancel - returns 0
void Size_Dialog::Cancel(wxCommandEvent& event)
{
  EndModal(0);
}
